﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;
using HeuristicLab.PluginInfrastructure;
using HeuristicLab.Problems.Instances;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment.Views {
  [View("GeneralizedQuadraticAssignmentProblemView")]
  [Content(typeof(GeneralizedQuadraticAssignmentProblem), IsDefaultView = true)]
  public partial class GeneralizedQuadraticAssignmentProblemView : ParameterizedNamedItemView {

    public new GeneralizedQuadraticAssignmentProblem Content {
      get { return (GeneralizedQuadraticAssignmentProblem)base.Content; }
      set { base.Content = value; }
    }

    public GeneralizedQuadraticAssignmentProblemView() {
      InitializeComponent();
      problemInstanceProviderComboBox.DisplayMember = "Name";
      instancesComboBox.DisplayMember = "Name";
      problemInstanceProviderComboBox.DataSource = GetProblemInstanceProviders().ToList();
    }

    #region Register Content Events
    protected override void DeregisterContentEvents() {
      base.DeregisterContentEvents();
    }
    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
    }
    #endregion

    protected override void OnContentChanged() {
      base.OnContentChanged();
      problemInstanceProviderComboBox.DataSource = GetProblemInstanceProviders().ToList();
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
    }

    #region Content Event Handlers
    #endregion

    #region Event Handlers
    private void problemInstanceProviderComboBox_SelectedIndexChanged(object sender, System.EventArgs e) {
      var provider = (IProblemInstanceProvider)problemInstanceProviderComboBox.SelectedItem;
      instancesComboBox.DataSource = provider.GetInstanceDescriptors().ToList();
    }

    private void comboBox_DataSourceChanged(object sender, EventArgs e) {
      var comboBox = (ComboBox)sender;
      if (comboBox.DataSource == null)
        comboBox.Items.Clear();
    }

    private void loadButton_Click(object sender, EventArgs e) {
      var provider = (IProblemInstanceProvider)problemInstanceProviderComboBox.SelectedItem;
      var descriptor = (IInstanceDescriptor)instancesComboBox.SelectedItem;
      OnLoadProblemInstanceButtonClicked(provider, descriptor);
    }
    #endregion

    #region Helpers
    #endregion

    protected virtual IEnumerable<IProblemInstanceProvider> GetProblemInstanceProviders() {
      var qapProviders = ApplicationManager.Manager.GetInstances<IProblemInstanceProvider<IQAPInstance>>().Cast<IProblemInstanceProvider>();
      var ctapProviders = ApplicationManager.Manager.GetInstances<IProblemInstanceProvider<ICTAPInstance>>().Cast<IProblemInstanceProvider>();
      return qapProviders.Concat(ctapProviders);
    }

    protected virtual void OnLoadProblemInstanceButtonClicked(IProblemInstanceProvider provider, IInstanceDescriptor descriptor) {
      if (provider is IProblemInstanceProvider<IQAPInstance>) {
        Content.LoadFrom(((IProblemInstanceProvider<IQAPInstance>)provider).GetInstance(descriptor));
      } else if (provider is IProblemInstanceProvider<ICTAPInstance>) {
        Content.LoadFrom(((IProblemInstanceProvider<ICTAPInstance>)provider).GetInstance(descriptor));
      }
    }
  }
}
